@@ -1,4 +1,4 @@
-/*	$OpenBSD: main.c,v 1.87 2019/06/22 15:38:15 lum Exp $	*/
+/*	$OpenBSD: main.c,v 1.95 2023/04/14 15:34:08 tb Exp $	*/
 
 /* This file is in the public domain. */
 
@@ -14,7 +14,15 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <termios.h>
 #include <unistd.h>
+#if defined(__linux__) || defined(__CYGWIN__)
+#include <pty.h>
+#elif defined(__OpenBSD__) || defined(__NetBSD__) || defined(__APPLE__)
+#include <util.h>
+#else
+#include <libutil.h>
+#endif
 
 #include "def.h"
 #include "kbd.h"
@@ -33,22 +41,26 @@ int		 doaudiblebell;			/* audible bell toggle	*/
 int		 dovisiblebell;			/* visible bell toggle	*/
 int		 dblspace;			/* sentence end #spaces	*/
 int		 allbro;			/* all buffs read-only	*/
+int		 batch;				/* for regress tests	*/
 struct buffer	*curbp;				/* current buffer	*/
 struct buffer	*bheadp;			/* BUFFER list head	*/
 struct mgwin	*curwp;				/* current window	*/
 struct mgwin	*wheadp;			/* MGWIN listhead	*/
+struct vhead	 varhead;			/* Variable list head	*/
 char		 pat[NPAT];			/* pattern		*/
 
 static void	 edinit(struct buffer *);
+static void	 pty_init(void);
 static __dead void usage(void);
 
 extern char	*__progname;
 extern void     closetags(void);
 
 static __dead void
-usage()
+usage(void)
 {
-	fprintf(stderr, "usage: %s [-nR] [-f mode] [+number] [file ...]\n",
+	fprintf(stderr, "usage: %s [-nR] [-b file] [-f mode] [-u file] "
+	    "[+number] [file ...]\n",
 	    __progname);
 	exit(1);
 }
@@ -56,7 +68,10 @@ usage()
 int
 main(int argc, char **argv)
 {
-	char		*cp, *init_fcn_name = NULL;
+	FILE		*ffp;
+	char		 file[NFILEN];
+	char		*cp, *conffile = NULL, *init_fcn_name = NULL;
+	char		*batchfile = NULL;
 	PF		 init_fcn = NULL;
 	int	 	 o, i, nfiles;
 	int	  	 nobackups = 0;
@@ -68,8 +83,12 @@ main(int argc, char **argv)
 		err(1, "pledge");
 #endif
 
-	while ((o = getopt(argc, argv, "nRf:")) != -1)
+	while ((o = getopt(argc, argv, "nRb:f:u:")) != -1)
 		switch (o) {
+		case 'b':
+			batch = 1;
+			batchfile = optarg;
+			break;
 		case 'R':
 			allbro = 1;
 			break;
@@ -82,9 +101,29 @@ main(int argc, char **argv)
 				    "initial function");
 			init_fcn_name = optarg;
 			break;
+		case 'u':
+			conffile = optarg;
+			break;
 		default:
 			usage();
 		}
+
+	if (batch && (conffile != NULL)) {
+                fprintf(stderr, "%s: -b and -u are mutually exclusive.\n",
+                    __progname);
+                exit(1);
+	}
+	if (batch) {
+		pty_init();
+		conffile = batchfile;
+	}
+	if ((ffp = startupfile(NULL, conffile, file, sizeof(file))) == NULL &&
+	    conffile != NULL) {
+		fprintf(stderr, "%s: Problem with file: %s\n", __progname,
+		    conffile);
+		exit(1);
+	}
+
 	argc -= optind;
 	argv += optind;
 
@@ -131,8 +170,13 @@ main(int argc, char **argv)
 	update(CMODE);
 
 	/* user startup file. */
-	if ((cp = startupfile(NULL)) != NULL)
-		(void)load(cp);
+	if (ffp != NULL) {
+		(void)load(ffp, file);
+		ffclose(ffp, NULL);
+	}
+
+	if (batch)
+		return (0);
 
 	/*
 	 * Now ensure any default buffer modes from the startup file are
@@ -249,11 +293,30 @@ edinit(struct buffer *bp)
 	wp->w_rflag = WFMODE | WFFULL;		/* Full.		 */
 }
 
+/*
+ * Create pty for batch mode.
+ */
+static void
+pty_init(void)
+{
+	struct winsize	 ws;
+	int		 master;
+	int		 slave;
+
+	memset(&ws, 0, sizeof(ws));
+	ws.ws_col = 80,
+	ws.ws_row = 24;
+
+	openpty(&master, &slave, NULL, NULL, &ws);
+	login_tty(slave);
+
+	return;
+}
+
 /*
  * Quit command.  If an argument, always quit.  Otherwise confirm if a buffer
  * has been changed and not written out.  Normally bound to "C-x C-c".
  */
-/* ARGSUSED */
 int
 quit(int f, int n)
 {
@@ -276,7 +339,6 @@ quit(int f, int n)
  * User abort.  Should be called by any input routine that sees a C-g to abort
  * whatever C-g is aborting these days. Currently does nothing.
  */
-/* ARGSUSED */
 int
 ctrlg(int f, int n)
 {
